@@ -278,13 +278,6 @@ public:
 
   nsresult Init();
 
-  /**
-   * This is expected to be called off the critical path to content
-   * startup.  This is an opportunity to load things that are slow
-   * on the critical path.
-   */
-  static void PreloadSlowThings();
-
   /** Return a TabChild with the given attributes. */
   static already_AddRefed<TabChild>
   Create(nsIContentChild* aManager, const TabId& aTabId,
@@ -699,9 +692,7 @@ private:
 
   void ActorDestroy(ActorDestroyReason why) override;
 
-  enum FrameScriptLoading { DONT_LOAD_SCRIPTS, DEFAULT_LOAD_SCRIPTS };
-
-  bool InitTabChildGlobal(FrameScriptLoading aScriptLoading = DEFAULT_LOAD_SCRIPTS);
+  bool InitTabChildGlobal();
 
   bool InitRenderingState(const TextureFactoryIdentifier& aTextureFactoryIdentifier,
                           const uint64_t& aLayersId,
@@ -709,8 +700,6 @@ private:
 
   void DestroyWindow();
 
-  void SetProcessNameToAppName();
-
   void ApplyShowInfo(const ShowInfo& aInfo);
 
   bool HasValidInnerSize();
