@@ -437,18 +437,19 @@ CrossCompartmentWrapper::fun_toString(JSContext* cx, HandleObject wrapper, bool
 }
 
 bool
-CrossCompartmentWrapper::regexp_toShared(JSContext* cx, HandleObject wrapper, RegExpGuard* g) const
+CrossCompartmentWrapper::regexp_toShared(JSContext* cx, HandleObject wrapper,
+                                         MutableHandleRegExpShared shared) const
 {
-    RegExpGuard wrapperGuard(cx);
+    RootedRegExpShared re(cx);
     {
         AutoCompartment call(cx, wrappedObject(wrapper));
-        if (!Wrapper::regexp_toShared(cx, wrapper, &wrapperGuard))
+        if (!Wrapper::regexp_toShared(cx, wrapper, &re))
             return false;
     }
 
     // Get an equivalent RegExpShared associated with the current compartment.
-    RegExpShared* re = wrapperGuard.re();
-    return cx->compartment()->regExps.get(cx, re->getSource(), re->getFlags(), g);
+    RootedAtom source(cx, re->getSource());
+    return cx->compartment()->regExps.get(cx, source, re->getFlags(), shared);
 }
 
 bool
