@@ -555,13 +555,6 @@ FTPChannelParent::SetParentListener(HttpChannelParentListener* aListener)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-FTPChannelParent::NotifyTrackingProtectionDisabled()
-{
-  // One day, this should probably be filled in.
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 FTPChannelParent::Delete()
 {
